/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkRubyUtil.h,v $

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
#ifndef __vtkRubyUtil_h
#define __vtkRubyUtil_h

#include "vtkRuby.h"
#include "vtkCommand.h"

#if defined(WIN32)
# if defined(vtkCommonRubyD_EXPORTS)
#  define VTK_RUBY_EXPORT __declspec(dllexport)
# else
#  define VTK_RUBY_EXPORT __declspec(dllimport)
# endif
#else
# define VTK_RUBY_EXPORT
#endif

// This is the VTK/Ruby 'class,' it contains the method list and a pointer
// to the superclass
typedef vtkObjectBase *(*vtknewfunc)();

typedef struct {
  vtkObjectBase *vtk_ptr;
  int ref_count;
  VALUE mark;
} rbVTKObject;

// This for objects not derived from vtkObjectBase
typedef struct {
  void *ptr;
  int ref_count;
  VALUE mark;
} rbVTKSpecialObject;


extern VTK_RUBY_EXPORT
VALUE rb_VTKClass(const char*);

extern VTK_RUBY_EXPORT
VALUE rb_vtkObjectBaseClass(void);

extern VTK_RUBY_EXPORT
VALUE rb_vtk_define_class_under(VALUE module, const char* name, VALUE super);

// Standard methods for all vtk/ruby objects
extern VTK_RUBY_EXPORT
VALUE rb_VTKObject_New(VALUE vtkclass, vtkObjectBase *ptr);

extern VTK_RUBY_EXPORT
void rb_VTKObject_PushMark(VALUE self, VALUE other);



// Extract the vtkObjectBase from a rbVTKObject.  If the VALUE is not a 
// rbVTKObject, or is not a rbVTKObject of the specified type, the ruby
// error indicator will be set.
// Special behaviour: Qnil is converted to NULL without no error.
extern VTK_RUBY_EXPORT
vtkObjectBase *vtkRubyGetPointerFromObject(VALUE obj);

// Convert a vtkObjectBase to a rbVTKObject.  This will first check to see if
// the rbVTKObject already exists, and create a new rbVTKObject if necessary.
// This function also passes ownership of the reference to the VALUE.
// Special behaviour: NULL is converted to Qnil.
extern VTK_RUBY_EXPORT
VALUE vtkRubyGetObjectFromPointer(vtkObjectBase *ptr);
extern VTK_RUBY_EXPORT
VALUE vtkRubyGetObjectFromPointer(vtkObjectBase *ptr, VALUE klass);
extern VTK_RUBY_EXPORT
VALUE vtkRubyGetObjectFromObject(VALUE arg, const char *type, VALUE klass);

extern VTK_RUBY_EXPORT
void vtkRubyUnRegister(rbVTKObject *obj);
extern VTK_RUBY_EXPORT
void vtkRubyUnRegister(VALUE self);

extern VTK_RUBY_EXPORT
char *vtkRubyManglePointer(void *ptr, const char *type);
extern VTK_RUBY_EXPORT
void *vtkRubyUnmanglePointer(char *ptrText, int *len, const char *type);


// check array arguments sent through the wrappers to see if the underlying
// C++ method changed the values, and attempt to modify the original ruby
// sequence (list or tuple) if so.
extern VTK_RUBY_EXPORT
int vtkRubyCheckArray(VALUE *args, int i, char *a, int n);
extern VTK_RUBY_EXPORT
int vtkRubyCheckArray(VALUE *args, int i, signed char *a, int n);
extern VTK_RUBY_EXPORT
int vtkRubyCheckArray(VALUE *args, int i, unsigned char *a, int n);
extern VTK_RUBY_EXPORT
int vtkRubyCheckArray(VALUE *args, int i, short *a, int n);
extern VTK_RUBY_EXPORT
int vtkRubyCheckArray(VALUE *args, int i, unsigned short *a, int n);
extern VTK_RUBY_EXPORT
int vtkRubyCheckArray(VALUE *args, int i, int *a, int n);
extern VTK_RUBY_EXPORT
int vtkRubyCheckArray(VALUE *args, int i, unsigned int *a, int n);
extern VTK_RUBY_EXPORT
int vtkRubyCheckArray(VALUE *args, int i, long *a, int n);
extern VTK_RUBY_EXPORT
int vtkRubyCheckArray(VALUE *args, int i, unsigned long *a, int n);
extern VTK_RUBY_EXPORT
int vtkRubyCheckArray(VALUE *args, int i, float *a, int n);
extern VTK_RUBY_EXPORT
int vtkRubyCheckArray(VALUE *args, int i, double *a, int n);
#if defined(VTK_TYPE_USE_LONG_LONG)
extern VTK_RUBY_EXPORT
int vtkRubyCheckArray(VALUE *args, int i, long long *a, int n);
extern VTK_RUBY_EXPORT
int vtkRubyCheckArray(VALUE *args, int i, unsigned long long *a, int n);
#endif
#if defined(VTK_TYPE_USE___INT64)
extern VTK_RUBY_EXPORT
int vtkRubyCheckArray(VALUE *args, int i, __int64 *a, int n);
extern VTK_RUBY_EXPORT
int vtkRubyCheckArray(VALUE *args, int i, unsigned __int64 *a, int n);
#endif

// For use by SetXXMethod() , SetXXMethodArgDelete()
extern VTK_RUBY_EXPORT void vtkRubyVoidFunc(void *);
extern VTK_RUBY_EXPORT void vtkRubyVoidFuncArgDelete(void *);

// To allow Ruby to use the vtkCommand features
class vtkRubyCommand : public vtkCommand
{
public:
  static vtkRubyCommand *New() { return new vtkRubyCommand; };

  void SetObject(VALUE o);
  void Execute(vtkObject *ptr, unsigned long eventtype, void *CallData);
 
  VALUE obj;
protected:
  vtkRubyCommand();
  ~vtkRubyCommand(); 
};

#endif
